# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .ja_4_fingerprint_type_id import Ja4FingerprintTypeId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Ja4Fingerprint(UncheckedBaseModel):
    """
    The JA4+ fingerprint object provides detailed fingerprint information about various aspects of network traffic which is both machine and human readable.
    """

    section_a: typing.Optional[str] = pydantic.Field(default=None)
    """
    The 'a' section of the JA4 fingerprint.
    """

    section_b: typing.Optional[str] = pydantic.Field(default=None)
    """
    The 'b' section of the JA4 fingerprint.
    """

    section_c: typing.Optional[str] = pydantic.Field(default=None)
    """
    The 'c' section of the JA4 fingerprint.
    """

    section_d: typing.Optional[str] = pydantic.Field(default=None)
    """
    The 'd' section of the JA4 fingerprint.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The JA4+ fingerprint type as defined by <a href='https://blog.foxio.io/ja4+-network-fingerprinting target='\_blank'>FoxIO</a>, normalized to the caption of 'type_id'. In the case of 'Other', it is defined by the event source.
    """

    type_id: Ja4FingerprintTypeId = pydantic.Field()
    """
    The identifier of the JA4+ fingerprint type.
    """

    value: str = pydantic.Field()
    """
    The JA4+ fingerprint value.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AwsRegion(str, enum.Enum):
    US_EAST_1 = "us-east-1"
    """
    US East 1 (N. Virginia)
    """

    US_EAST_2 = "us-east-2"
    """
    US East 2 (Ohio)
    """

    US_WEST_1 = "us-west-1"
    """
    US West 1 (N. California)
    """

    US_WEST_2 = "us-west-2"
    """
    US West 2 (Oregon)
    """

    AF_SOUTH_1 = "af-south-1"
    """
    Africa South 1 (Cape Town)
    """

    AP_EAST_1 = "ap-east-1"
    """
    Asia Pacific East 1 (Hong Kong)
    """

    AP_SOUTH_2 = "ap-south-2"
    """
    Asia Pacific South 2 (Hyderabad)
    """

    AP_SOUTHEAST_3 = "ap-southeast-3"
    """
    Asia Pacific Southeast 3 (Jakarta)
    """

    AP_SOUTHEAST_5 = "ap-southeast-5"
    """
    Asia Pacific Southeast 5 (Malaysia)
    """

    AP_SOUTHEAST_4 = "ap-southeast-4"
    """
    Asia Pacific Southeast 4 (Melbourne)
    """

    AP_SOUTH_1 = "ap-south-1"
    """
    Asia Pacific South 1 (Mumbai)
    """

    AP_NORTHEAST_3 = "ap-northeast-3"
    """
    Asia Pacific Northeast 3 (Osaka)
    """

    AP_NORTHEAST_2 = "ap-northeast-2"
    """
    Asia Pacific Northeast 2 (Seoul)
    """

    AP_SOUTHEAST_1 = "ap-southeast-1"
    """
    Asia Pacific Southeast 1 (Singapore)
    """

    AP_SOUTHEAST_2 = "ap-southeast-2"
    """
    Asia Pacific Southeast 2 (Sydney)
    """

    AP_EAST_2 = "ap-east-2"
    """
    Asia Pacific East 2 (Taipei)
    """

    AP_SOUTHEAST_7 = "ap-southeast-7"
    """
    Asia Pacific Southeast 7 (Thailand)
    """

    AP_NORTHEAST_1 = "ap-northeast-1"
    """
    Asia Pacific Northeast 1 (Tokyo)
    """

    CA_CENTRAL_1 = "ca-central-1"
    """
    Canada Central 1 (Central)
    """

    CA_WEST_1 = "ca-west-1"
    """
    Canada West 1 (Calgary)
    """

    EU_CENTRAL_1 = "eu-central-1"
    """
    Europe Central 1 (Frankfurt)
    """

    EU_WEST_1 = "eu-west-1"
    """
    Europe West 1 (Ireland)
    """

    EU_WEST_2 = "eu-west-2"
    """
    Europe West 2 (London)
    """

    EU_SOUTH_1 = "eu-south-1"
    """
    Europe South 1 (Milan)
    """

    EU_WEST_3 = "eu-west-3"
    """
    Europe West 3 (Paris)
    """

    EU_SOUTH_2 = "eu-south-2"
    """
    Europe South 2 (Spain)
    """

    EU_NORTH_1 = "eu-north-1"
    """
    Europe North 1 (Stockholm)
    """

    EU_CENTRAL_2 = "eu-central-2"
    """
    Europe Central 2 (Zurich)
    """

    IL_CENTRAL_1 = "il-central-1"
    """
    Israel Central 1 (Tel Aviv)
    """

    MX_CENTRAL_1 = "mx-central-1"
    """
    Mexico Central 1
    """

    ME_SOUTH_1 = "me-south-1"
    """
    Middle East South 1 (Bahrain)
    """

    ME_CENTRAL_1 = "me-central-1"
    """
    Middle East Central 1 (UAE)
    """

    SA_EAST_1 = "sa-east-1"
    """
    South America East 1 (São Paulo)
    """

    _UNKNOWN = "__AWSREGION_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "AwsRegion":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        us_east_1: typing.Callable[[], T_Result],
        us_east_2: typing.Callable[[], T_Result],
        us_west_1: typing.Callable[[], T_Result],
        us_west_2: typing.Callable[[], T_Result],
        af_south_1: typing.Callable[[], T_Result],
        ap_east_1: typing.Callable[[], T_Result],
        ap_south_2: typing.Callable[[], T_Result],
        ap_southeast_3: typing.Callable[[], T_Result],
        ap_southeast_5: typing.Callable[[], T_Result],
        ap_southeast_4: typing.Callable[[], T_Result],
        ap_south_1: typing.Callable[[], T_Result],
        ap_northeast_3: typing.Callable[[], T_Result],
        ap_northeast_2: typing.Callable[[], T_Result],
        ap_southeast_1: typing.Callable[[], T_Result],
        ap_southeast_2: typing.Callable[[], T_Result],
        ap_east_2: typing.Callable[[], T_Result],
        ap_southeast_7: typing.Callable[[], T_Result],
        ap_northeast_1: typing.Callable[[], T_Result],
        ca_central_1: typing.Callable[[], T_Result],
        ca_west_1: typing.Callable[[], T_Result],
        eu_central_1: typing.Callable[[], T_Result],
        eu_west_1: typing.Callable[[], T_Result],
        eu_west_2: typing.Callable[[], T_Result],
        eu_south_1: typing.Callable[[], T_Result],
        eu_west_3: typing.Callable[[], T_Result],
        eu_south_2: typing.Callable[[], T_Result],
        eu_north_1: typing.Callable[[], T_Result],
        eu_central_2: typing.Callable[[], T_Result],
        il_central_1: typing.Callable[[], T_Result],
        mx_central_1: typing.Callable[[], T_Result],
        me_south_1: typing.Callable[[], T_Result],
        me_central_1: typing.Callable[[], T_Result],
        sa_east_1: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is AwsRegion.US_EAST_1:
            return us_east_1()
        if self is AwsRegion.US_EAST_2:
            return us_east_2()
        if self is AwsRegion.US_WEST_1:
            return us_west_1()
        if self is AwsRegion.US_WEST_2:
            return us_west_2()
        if self is AwsRegion.AF_SOUTH_1:
            return af_south_1()
        if self is AwsRegion.AP_EAST_1:
            return ap_east_1()
        if self is AwsRegion.AP_SOUTH_2:
            return ap_south_2()
        if self is AwsRegion.AP_SOUTHEAST_3:
            return ap_southeast_3()
        if self is AwsRegion.AP_SOUTHEAST_5:
            return ap_southeast_5()
        if self is AwsRegion.AP_SOUTHEAST_4:
            return ap_southeast_4()
        if self is AwsRegion.AP_SOUTH_1:
            return ap_south_1()
        if self is AwsRegion.AP_NORTHEAST_3:
            return ap_northeast_3()
        if self is AwsRegion.AP_NORTHEAST_2:
            return ap_northeast_2()
        if self is AwsRegion.AP_SOUTHEAST_1:
            return ap_southeast_1()
        if self is AwsRegion.AP_SOUTHEAST_2:
            return ap_southeast_2()
        if self is AwsRegion.AP_EAST_2:
            return ap_east_2()
        if self is AwsRegion.AP_SOUTHEAST_7:
            return ap_southeast_7()
        if self is AwsRegion.AP_NORTHEAST_1:
            return ap_northeast_1()
        if self is AwsRegion.CA_CENTRAL_1:
            return ca_central_1()
        if self is AwsRegion.CA_WEST_1:
            return ca_west_1()
        if self is AwsRegion.EU_CENTRAL_1:
            return eu_central_1()
        if self is AwsRegion.EU_WEST_1:
            return eu_west_1()
        if self is AwsRegion.EU_WEST_2:
            return eu_west_2()
        if self is AwsRegion.EU_SOUTH_1:
            return eu_south_1()
        if self is AwsRegion.EU_WEST_3:
            return eu_west_3()
        if self is AwsRegion.EU_SOUTH_2:
            return eu_south_2()
        if self is AwsRegion.EU_NORTH_1:
            return eu_north_1()
        if self is AwsRegion.EU_CENTRAL_2:
            return eu_central_2()
        if self is AwsRegion.IL_CENTRAL_1:
            return il_central_1()
        if self is AwsRegion.MX_CENTRAL_1:
            return mx_central_1()
        if self is AwsRegion.ME_SOUTH_1:
            return me_south_1()
        if self is AwsRegion.ME_CENTRAL_1:
            return me_central_1()
        if self is AwsRegion.SA_EAST_1:
            return sa_east_1()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from .credential_base import CredentialBase
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class OAuthClientCredential(CredentialBase):
    """
    A Client ID and secret used for authenticating with OAuth 2.0 compatible service using the client credentials grant.
    """

    token_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional URL for the OAuth 2.0 token exchange if it can not be constructed based on provider configuration
    """

    client_id: str = pydantic.Field()
    """
    The ID of the client application defined at the service provider
    """

    client_secret: str = pydantic.Field()
    """
    Secret value for authentication
    """

    extra: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = (
        pydantic.Field(default=None)
    )
    """
    Optional connection specific JSON map data such as a signing key ID or organization ID
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ParameterLocation(str, enum.Enum):
    HEADER = "header"
    PATH = "path"
    QUERY = "query"
    BODY = "body"
    _UNKNOWN = "__PARAMETERLOCATION_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ParameterLocation":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        header: typing.Callable[[], T_Result],
        path: typing.Callable[[], T_Result],
        query: typing.Callable[[], T_Result],
        body: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ParameterLocation.HEADER:
            return header()
        if self is ParameterLocation.PATH:
            return path()
        if self is ParameterLocation.QUERY:
            return query()
        if self is ParameterLocation.BODY:
            return body()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .remediation import Remediation
from .rule import Rule
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class CisBenchmarkResult(UncheckedBaseModel):
    """
    The CIS Benchmark Result object contains information as defined by the Center for Internet Security (<a target='_blank' href='https://www.cisecurity.org/cis-benchmarks/'>CIS</a>) benchmark result. CIS Benchmarks are a collection of best practices for securely configuring IT systems, software, networks, and cloud infrastructure.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The CIS benchmark description.
    """

    name: str = pydantic.Field()
    """
    The CIS benchmark name.
    """

    remediation: typing.Optional[Remediation] = pydantic.Field(default=None)
    """
    Describes the recommended remediation steps to address identified issue(s).
    """

    rule: typing.Optional[Rule] = pydantic.Field(default=None)
    """
    The CIS benchmark rule.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.unchecked_base_model import UncheckedBaseModel
import typing
import typing_extensions
from ...core.unchecked_base_model import UnionMetadata


class BridgeSelector_Id(UncheckedBaseModel):
    value: str
    type: typing.Literal["id"] = "id"

    class Config:
        smart_union = True


class BridgeSelector_Labels(UncheckedBaseModel):
    value: typing.List[str]
    type: typing.Literal["labels"] = "labels"

    class Config:
        smart_union = True


BridgeSelector = typing_extensions.Annotated[
    typing.Union[BridgeSelector_Id, BridgeSelector_Labels],
    UnionMetadata(discriminant="type"),
]



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class AutonomousSystem(UncheckedBaseModel):
    """
    An autonomous system (AS) is a collection of connected Internet Protocol (IP) routing prefixes under the control of one or more network operators on behalf of a single administrative entity or domain that presents a common, clearly defined routing policy to the internet.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Organization name for the Autonomous System.
    """

    number: typing.Optional[int] = pydantic.Field(default=None)
    """
    Unique number that the AS is identified by.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .remote_field_type_id import RemoteFieldTypeId
import typing
import pydantic
from ...engine.types.option_value import OptionValue
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class RemoteFieldSchema(UncheckedBaseModel):
    field_type_id: RemoteFieldTypeId
    field_type: typing.Optional[str] = pydantic.Field(alias="field_Type", default=None)
    enum_values: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Display names for possible values the enum can take on.
    """

    enum_option_values: typing.Optional[typing.List[OptionValue]] = pydantic.Field(
        default=None
    )
    """
    Possible values the enum can take on. For providers which don't distinguish between ID and display name, the ID and display name should be set to the same value.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from ...base.types.timestamp import Timestamp
import pydantic
import datetime as dt
from .group import Group
from .database_type_id import DatabaseTypeId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Database(UncheckedBaseModel):
    """
    The database object is used for databases which are typically datastore services that contain an organized collection of structured and unstructured data or a types of data.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the database was known to have been created.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the database was known to have been created.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the database.
    """

    groups: typing.Optional[typing.List[Group]] = pydantic.Field(default=None)
    """
    The group names to which the database belongs.
    """

    modified_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The most recent time when any changes, updates, or modifications were made within the database.
    """

    modified_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The most recent time when any changes, updates, or modifications were made within the database.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The database name, ordinarily as assigned by a database administrator.
    """

    size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The size of the database in bytes.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The database type.
    """

    type_id: DatabaseTypeId = pydantic.Field()
    """
    The normalized identifier of the database type.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the database.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .types.notification_id import NotificationId
from ..core.request_options import RequestOptions
from .types.get_notification_response import GetNotificationResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.unchecked_base_model import construct_type
from ..common.errors.bad_request_error import BadRequestError
from ..common.types.problem import Problem
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.internal_server_error import InternalServerError
from ..common.errors.not_implemented_error import NotImplementedError
from ..common.errors.bad_gateway_error import BadGatewayError
from ..common.errors.service_unavailable_error import ServiceUnavailableError
from ..common.errors.gateway_timeout_error import GatewayTimeoutError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..ticketing.types.priority import Priority
from .types.create_notification_response import CreateNotificationResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class NotificationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_message(
        self,
        notification_id: NotificationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetNotificationResponse:
        """
        Returns the `Notification` object matching `{notificationId}` from the token-linked `Integration`.

        Parameters
        ----------
        notification_id : NotificationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetNotificationResponse

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.notifications.get_message(
            notification_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/notifications/get/{jsonable_encoder(notification_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetNotificationResponse,
                    construct_type(
                        type_=GetNotificationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_message(
        self,
        *,
        summary: str,
        name: typing.Optional[str] = OMIT,
        priority: typing.Optional[Priority] = OMIT,
        project: typing.Optional[str] = OMIT,
        status: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        issue_type: typing.Optional[str] = OMIT,
        creator: typing.Optional[str] = OMIT,
        assignee: typing.Optional[str] = OMIT,
        contact: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateNotificationResponse:
        """
        Creates a `Notification` object in the token-linked `Integration`.

        Parameters
        ----------
        summary : str
            Notification summary.

        name : typing.Optional[str]
            This field is deprecated and no longer used.

        priority : typing.Optional[Priority]
            Notification priority

        project : typing.Optional[str]
            Notification project

        status : typing.Optional[str]
            The current status of the notification.

        description : typing.Optional[str]
            Notification description.

        issue_type : typing.Optional[str]
            The notification's type.

        creator : typing.Optional[str]
            The user who created this notification.

        assignee : typing.Optional[str]
            Who notification is assigned to.

        contact : typing.Optional[str]
            The notification contact information.

        tags : typing.Optional[typing.Sequence[str]]
            Associate tags with Notification

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateNotificationResponse

        Examples
        --------
        from synqly import SynqlyEngine
        from synqly.ticketing import Priority

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.notifications.create_message(
            name="string",
            summary="string",
            priority=Priority.URGENT,
            project="string",
            status="string",
            description="string",
            issue_type="string",
            creator="string",
            assignee="string",
            contact="string",
            tags=["string"],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/notifications/create",
            method="POST",
            json={
                "name": name,
                "summary": summary,
                "priority": priority,
                "project": project,
                "status": status,
                "description": description,
                "issue_type": issue_type,
                "creator": creator,
                "assignee": assignee,
                "contact": contact,
                "tags": tags,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateNotificationResponse,
                    construct_type(
                        type_=CreateNotificationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def clear_message(
        self,
        notification_id: NotificationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Resolves a `Notification` object in the token-linked `Integration`.

        Parameters
        ----------
        notification_id : NotificationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.notifications.clear_message(
            notification_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/notifications/clear/{jsonable_encoder(notification_id)}",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncNotificationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_message(
        self,
        notification_id: NotificationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetNotificationResponse:
        """
        Returns the `Notification` object matching `{notificationId}` from the token-linked `Integration`.

        Parameters
        ----------
        notification_id : NotificationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetNotificationResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.notifications.get_message(
                notification_id="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/notifications/get/{jsonable_encoder(notification_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetNotificationResponse,
                    construct_type(
                        type_=GetNotificationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_message(
        self,
        *,
        summary: str,
        name: typing.Optional[str] = OMIT,
        priority: typing.Optional[Priority] = OMIT,
        project: typing.Optional[str] = OMIT,
        status: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        issue_type: typing.Optional[str] = OMIT,
        creator: typing.Optional[str] = OMIT,
        assignee: typing.Optional[str] = OMIT,
        contact: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateNotificationResponse:
        """
        Creates a `Notification` object in the token-linked `Integration`.

        Parameters
        ----------
        summary : str
            Notification summary.

        name : typing.Optional[str]
            This field is deprecated and no longer used.

        priority : typing.Optional[Priority]
            Notification priority

        project : typing.Optional[str]
            Notification project

        status : typing.Optional[str]
            The current status of the notification.

        description : typing.Optional[str]
            Notification description.

        issue_type : typing.Optional[str]
            The notification's type.

        creator : typing.Optional[str]
            The user who created this notification.

        assignee : typing.Optional[str]
            Who notification is assigned to.

        contact : typing.Optional[str]
            The notification contact information.

        tags : typing.Optional[typing.Sequence[str]]
            Associate tags with Notification

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateNotificationResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine
        from synqly.ticketing import Priority

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.notifications.create_message(
                name="string",
                summary="string",
                priority=Priority.URGENT,
                project="string",
                status="string",
                description="string",
                issue_type="string",
                creator="string",
                assignee="string",
                contact="string",
                tags=["string"],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/notifications/create",
            method="POST",
            json={
                "name": name,
                "summary": summary,
                "priority": priority,
                "project": project,
                "status": status,
                "description": description,
                "issue_type": issue_type,
                "creator": creator,
                "assignee": assignee,
                "contact": contact,
                "tags": tags,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateNotificationResponse,
                    construct_type(
                        type_=CreateNotificationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def clear_message(
        self,
        notification_id: NotificationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Resolves a `Notification` object in the token-linked `Integration`.

        Parameters
        ----------
        notification_id : NotificationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.notifications.clear_message(
                notification_id="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/notifications/clear/{jsonable_encoder(notification_id)}",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import pydantic
from .upload_scan_status import UploadScanStatus
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ScanStatus(UncheckedBaseModel):
    scan_id: str = pydantic.Field()
    """
    ID of the scan.
    """

    status: UploadScanStatus = pydantic.Field()
    """
    Specifies the status of the scan, which can be Complete, Pending, or Error.
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Provider specific status message for this scan.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ...base.types.timestamp import Timestamp
import datetime as dt
from ...base.types.process_name import ProcessName
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class ProcessEntity(UncheckedBaseModel):
    """
    The Process Entity object provides critical fields for referencing a process.
    """

    cmd_line: typing.Optional[str] = pydantic.Field(default=None)
    """
    The full command line used to launch an application, service, process, or job. For example: <code>ssh user@10.0.0.10</code>. If the command line is unavailable or missing, the empty string <code>''</code> is to be used.
    """

    cpid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique process identifier that can be assigned deterministically by multiple system data producers.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the process was created/started.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the process was created/started.
    """

    name: typing.Optional[ProcessName] = pydantic.Field(default=None)
    """
    The friendly name of the process, for example: <code>Notepad++</code>.
    """

    path: typing.Optional[str] = pydantic.Field(default=None)
    """
    The process file path.
    """

    pid: typing.Optional[int] = pydantic.Field(default=None)
    """
    The process identifier, as reported by the operating system. Process ID (PID) is a number used by the operating system to uniquely identify an active process.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique identifier for this process assigned by the producer (tool). Facilitates correlation of a process event with other events for that process.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .patch_op import PatchOp
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class PatchOperation(UncheckedBaseModel):
    """
    JSON patch to apply. A JSON patch is a list of patch operations. (see https://jsonpatch.com/)
    """

    op: PatchOp = pydantic.Field()
    """
    The operation to perform. Supported values are `add`, `copy`, `move`, `replace`, `remove`, and `test`.
    """

    path: str = pydantic.Field()
    """
    The path to the field to update. The path is a JSON Pointer.
    """

    from_: typing.Optional[str] = pydantic.Field(alias="from", default=None)
    """
    The path to the field to copy from. This is required for `copy` and `move` operations.
    """

    value: typing.Optional[typing.Optional[typing.Any]] = pydantic.Field(default=None)
    """
    The value to set the field to. This is required for `add`, `replace` and `test` operations.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class SiemMock(UncheckedBaseModel):
    """
    Configuration for the Synqly mock in-memory SIEM Provider. This provider is for testing purposes only and does not retain events pushed to it.
    """

    index: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the index where events are stored.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Idp(UncheckedBaseModel):
    """
    The Identity Provider object contains detailed information about a provider responsible for creating, maintaining, and managing identity information while offering authentication services to applications. An Identity Provider (IdP) serves as a trusted authority that verifies the identity of users and issues authentication tokens or assertions to enable secure access to applications or services.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the identity provider.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the identity provider.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from ...common.types.id import Id
from ...engine.types.object import Object
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
import typing_extensions
from ...core.unchecked_base_model import UnionMetadata


class Actor_Agent(UncheckedBaseModel):
    """
    The actor object either relates gives a summary of an actor in an escalation policy or gives full details of an actor. This depends on the capabilities of the ticketing provider. An actor is someone or something that will be notified when certain conditions are met. It can either be an agent, a group, or a schedule.
    """

    type: typing.Literal["agent"] = "agent"
    uid: Id
    full_name: str
    email_addr: typing.Optional[str] = None
    phone_number: typing.Optional[str] = None
    time_zone: typing.Optional[str] = None
    job_title: typing.Optional[str] = None
    unmapped: typing.Optional[Object] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class Actor_AgentSummary(UncheckedBaseModel):
    """
    The actor object either relates gives a summary of an actor in an escalation policy or gives full details of an actor. This depends on the capabilities of the ticketing provider. An actor is someone or something that will be notified when certain conditions are met. It can either be an agent, a group, or a schedule.
    """

    type: typing.Literal["agent_summary"] = "agent_summary"
    id: Id
    name: str
    additional_properties: bool

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class Actor_GroupSummary(UncheckedBaseModel):
    """
    The actor object either relates gives a summary of an actor in an escalation policy or gives full details of an actor. This depends on the capabilities of the ticketing provider. An actor is someone or something that will be notified when certain conditions are met. It can either be an agent, a group, or a schedule.
    """

    type: typing.Literal["group_summary"] = "group_summary"
    id: Id
    name: typing.Optional[str] = None
    additional_properties: bool

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class Actor_ScheduleSummary(UncheckedBaseModel):
    """
    The actor object either relates gives a summary of an actor in an escalation policy or gives full details of an actor. This depends on the capabilities of the ticketing provider. An actor is someone or something that will be notified when certain conditions are met. It can either be an agent, a group, or a schedule.
    """

    type: typing.Literal["schedule_summary"] = "schedule_summary"
    id: Id
    name: typing.Optional[str] = None
    additional_properties: bool

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


Actor = typing_extensions.Annotated[
    typing.Union[
        Actor_Agent, Actor_AgentSummary, Actor_GroupSummary, Actor_ScheduleSummary
    ],
    UnionMetadata(discriminant="type"),
]



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .device import Device
import pydantic
from ...base.types.email_address import EmailAddress
from .auth_factor_factor_type_id import AuthFactorFactorTypeId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class AuthFactor(UncheckedBaseModel):
    """
    An Authentication Factor object describes a category of methods used for identity verification in an authentication attempt.
    """

    device: typing.Optional[Device] = pydantic.Field(default=None)
    """
    Device used to complete an authentication request.
    """

    email_addr: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The email address used in an email-based authentication factor.
    """

    factor_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of authentication factor used in an authentication attempt.
    """

    factor_type_id: AuthFactorFactorTypeId = pydantic.Field()
    """
    The normalized identifier for the authentication factor.
    """

    is_hotp: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the authentication factor is an HMAC-based One-time Password (HOTP).
    """

    is_totp: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the authentication factor is a Time-based One-time Password (TOTP).
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The phone number used for a telephony-based authentication request.
    """

    provider: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of provider for an authentication factor.
    """

    security_questions: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The question(s) provided to user for a question-based authentication factor.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .dns_answer_flag_ids import DnsAnswerFlagIds
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class DnsAnswer(UncheckedBaseModel):
    """
    The DNS Answer object represents a specific response provided by the Domain Name System (DNS) when querying for information about a domain or performing a DNS operation. It encapsulates the relevant details and data returned by the DNS server in response to a query.
    """

    class_: typing.Optional[str] = pydantic.Field(alias="class", default=None)
    """
    The class of DNS data contained in this resource record. See <a target='_blank' href='https://www.rfc-editor.org/rfc/rfc1035.txt'>RFC1035</a>. For example: <code>IN</code>.
    """

    flag_ids: typing.Optional[typing.List[DnsAnswerFlagIds]] = pydantic.Field(
        default=None
    )
    """
    The list of DNS answer header flag IDs.
    """

    flags: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of DNS answer header flags.
    """

    packet_uid: typing.Optional[int] = pydantic.Field(default=None)
    """
    The DNS packet identifier assigned by the program that generated the query. The identifier is copied to the response.
    """

    rdata: str = pydantic.Field()
    """
    The data describing the DNS resource. The meaning of this data depends on the type and class of the resource record.
    """

    ttl: typing.Optional[int] = pydantic.Field(default=None)
    """
    The time interval that the resource record may be cached. Zero value means that the resource record can only be used for the transaction in progress, and should not be cached.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of data contained in this resource record. See <a target='_blank' href='https://www.rfc-editor.org/rfc/rfc1035.txt'>RFC1035</a>. For example: <code>CNAME</code>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class OperationId(str, enum.Enum):
    APPSEC_GET_APPLICATION_FINDING_DETAILS = "appsec_get_application_finding_details"
    APPSEC_QUERY_APPLICATION_FINDINGS = "appsec_query_application_findings"
    APPSEC_QUERY_APPLICATIONS = "appsec_query_applications"
    APPSEC_QUERY_FINDINGS = "appsec_query_findings"
    ASSETS_CREATE_ASSET = "assets_create_asset"
    ASSETS_GET_LABELS = "assets_get_labels"
    ASSETS_QUERY_DEVICES = "assets_query_devices"
    CLOUDSECURITY_QUERY_CLOUD_RESOURCE_INVENTORY = (
        "cloudsecurity_query_cloud_resource_inventory"
    )
    CLOUDSECURITY_QUERY_COMPLIANCE_FINDINGS = "cloudsecurity_query_compliance_findings"
    CLOUDSECURITY_QUERY_EVENTS = "cloudsecurity_query_events"
    CLOUDSECURITY_QUERY_IOMS = "cloudsecurity_query_ioms"
    CLOUDSECURITY_QUERY_THREATS = "cloudsecurity_query_threats"
    CUSTOM_DELETE = "custom_delete"
    CUSTOM_GET = "custom_get"
    CUSTOM_PATCH = "custom_patch"
    CUSTOM_POST = "custom_post"
    CUSTOM_POST_BATCH = "custom_post_batch"
    CUSTOM_QUERY = "custom_query"
    EDR_CREATE_IOCS = "edr_create_iocs"
    EDR_DELETE_IOCS = "edr_delete_iocs"
    EDR_GET_ENDPOINT = "edr_get_endpoint"
    EDR_NETWORK_QUARANTINE = "edr_network_quarantine"
    EDR_QUERY_ALERTS = "edr_query_alerts"
    EDR_QUERY_APPLICATIONS = "edr_query_applications"
    EDR_QUERY_EDR_EVENTS = "edr_query_edr_events"
    EDR_QUERY_ENDPOINTS = "edr_query_endpoints"
    EDR_QUERY_IOCS = "edr_query_iocs"
    EDR_QUERY_POSTURE_SCORE = "edr_query_posture_score"
    EDR_QUERY_THREATEVENTS = "edr_query_threatevents"
    IDENTITY_DISABLE_USER = "identity_disable_user"
    IDENTITY_ENABLE_USER = "identity_enable_user"
    IDENTITY_EXPIRE_ALL_USER_SESSIONS = "identity_expire_all_user_sessions"
    IDENTITY_FORCE_USER_PASSWORD_RESET = "identity_force_user_password_reset"
    IDENTITY_GET_GROUP = "identity_get_group"
    IDENTITY_GET_GROUP_MEMBERS = "identity_get_group_members"
    IDENTITY_GET_USER = "identity_get_user"
    IDENTITY_GET_USER_PICTURE = "identity_get_user_picture"
    IDENTITY_QUERY_AUDIT_LOG = "identity_query_audit_log"
    IDENTITY_QUERY_GROUPS = "identity_query_groups"
    IDENTITY_QUERY_USERS = "identity_query_users"
    INCIDENTRESPONSE_QUERY_ESCALATION_POLICIES = (
        "incidentresponse_query_escalation_policies"
    )
    INCIDENTRESPONSE_QUERY_ESCALATION_POLICY_USERS_ON_CALL = (
        "incidentresponse_query_escalation_policy_users_on_call"
    )
    NOTIFICATIONS_CLEAR_MESSAGE = "notifications_clear_message"
    NOTIFICATIONS_CREATE_MESSAGE = "notifications_create_message"
    NOTIFICATIONS_GET_MESSAGE = "notifications_get_message"
    SIEM_GET_EVIDENCE = "siem_get_evidence"
    SIEM_GET_INVESTIGATION = "siem_get_investigation"
    SIEM_PATCH_INVESTIGATION = "siem_patch_investigation"
    SIEM_POST_EVENTS = "siem_post_events"
    SIEM_QUERY_ALERTS = "siem_query_alerts"
    SIEM_QUERY_EVENTS = "siem_query_events"
    SIEM_QUERY_INVESTIGATIONS = "siem_query_investigations"
    SIEM_QUERY_LOG_PROVIDERS = "siem_query_log_providers"
    SINK_POST_EVENTS = "sink_post_events"
    STORAGE_DELETE_FILE = "storage_delete_file"
    STORAGE_DOWNLOAD_FILE = "storage_download_file"
    STORAGE_LIST_FILES = "storage_list_files"
    STORAGE_UPLOAD_FILE = "storage_upload_file"
    TICKETING_CREATE_ATTACHMENT = "ticketing_create_attachment"
    TICKETING_CREATE_COMMENT = "ticketing_create_comment"
    TICKETING_CREATE_NOTE = "ticketing_create_note"
    TICKETING_CREATE_TICKET = "ticketing_create_ticket"
    TICKETING_DELETE_ATTACHMENT = "ticketing_delete_attachment"
    TICKETING_DELETE_COMMENT = "ticketing_delete_comment"
    TICKETING_DELETE_NOTE = "ticketing_delete_note"
    TICKETING_DOWNLOAD_ATTACHMENT = "ticketing_download_attachment"
    TICKETING_GET_TICKET = "ticketing_get_ticket"
    TICKETING_LIST_ATTACHMENTS_METADATA = "ticketing_list_attachments_metadata"
    TICKETING_LIST_COMMENTS = "ticketing_list_comments"
    TICKETING_LIST_NOTES = "ticketing_list_notes"
    TICKETING_LIST_ON_CALL = "ticketing_list_on_call"
    TICKETING_LIST_PROJECTS = "ticketing_list_projects"
    TICKETING_LIST_REMOTE_FIELDS = "ticketing_list_remote_fields"
    TICKETING_PATCH_NOTE = "ticketing_patch_note"
    TICKETING_PATCH_TICKET = "ticketing_patch_ticket"
    TICKETING_QUERY_ESCALATION_POLICIES = "ticketing_query_escalation_policies"
    TICKETING_QUERY_TICKETS = "ticketing_query_tickets"
    VULNERABILITIES_CREATE_ASSET = "vulnerabilities_create_asset"
    VULNERABILITIES_CREATE_FINDINGS = "vulnerabilities_create_findings"
    VULNERABILITIES_GET_LABELS = "vulnerabilities_get_labels"
    VULNERABILITIES_GET_SCAN_ACTIVITY = "vulnerabilities_get_scan_activity"
    VULNERABILITIES_GET_SCAN_STATUS = "vulnerabilities_get_scan_status"
    VULNERABILITIES_QUERY_ASSETS = "vulnerabilities_query_assets"
    VULNERABILITIES_QUERY_FINDINGS = "vulnerabilities_query_findings"
    VULNERABILITIES_QUERY_SCANS = "vulnerabilities_query_scans"
    VULNERABILITIES_UPDATE_ASSET = "vulnerabilities_update_asset"
    VULNERABILITIES_UPDATE_FINDING = "vulnerabilities_update_finding"
    VULNERABILITIES_UPLOAD_SCAN = "vulnerabilities_upload_scan"
    _UNKNOWN = "__OPERATIONID_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "OperationId":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        appsec_get_application_finding_details: typing.Callable[[], T_Result],
        appsec_query_application_findings: typing.Callable[[], T_Result],
        appsec_query_applications: typing.Callable[[], T_Result],
        appsec_query_findings: typing.Callable[[], T_Result],
        assets_create_asset: typing.Callable[[], T_Result],
        assets_get_labels: typing.Callable[[], T_Result],
        assets_query_devices: typing.Callable[[], T_Result],
        cloudsecurity_query_cloud_resource_inventory: typing.Callable[[], T_Result],
        cloudsecurity_query_compliance_findings: typing.Callable[[], T_Result],
        cloudsecurity_query_events: typing.Callable[[], T_Result],
        cloudsecurity_query_ioms: typing.Callable[[], T_Result],
        cloudsecurity_query_threats: typing.Callable[[], T_Result],
        custom_delete: typing.Callable[[], T_Result],
        custom_get: typing.Callable[[], T_Result],
        custom_patch: typing.Callable[[], T_Result],
        custom_post: typing.Callable[[], T_Result],
        custom_post_batch: typing.Callable[[], T_Result],
        custom_query: typing.Callable[[], T_Result],
        edr_create_iocs: typing.Callable[[], T_Result],
        edr_delete_iocs: typing.Callable[[], T_Result],
        edr_get_endpoint: typing.Callable[[], T_Result],
        edr_network_quarantine: typing.Callable[[], T_Result],
        edr_query_alerts: typing.Callable[[], T_Result],
        edr_query_applications: typing.Callable[[], T_Result],
        edr_query_edr_events: typing.Callable[[], T_Result],
        edr_query_endpoints: typing.Callable[[], T_Result],
        edr_query_iocs: typing.Callable[[], T_Result],
        edr_query_posture_score: typing.Callable[[], T_Result],
        edr_query_threatevents: typing.Callable[[], T_Result],
        identity_disable_user: typing.Callable[[], T_Result],
        identity_enable_user: typing.Callable[[], T_Result],
        identity_expire_all_user_sessions: typing.Callable[[], T_Result],
        identity_force_user_password_reset: typing.Callable[[], T_Result],
        identity_get_group: typing.Callable[[], T_Result],
        identity_get_group_members: typing.Callable[[], T_Result],
        identity_get_user: typing.Callable[[], T_Result],
        identity_get_user_picture: typing.Callable[[], T_Result],
        identity_query_audit_log: typing.Callable[[], T_Result],
        identity_query_groups: typing.Callable[[], T_Result],
        identity_query_users: typing.Callable[[], T_Result],
        incidentresponse_query_escalation_policies: typing.Callable[[], T_Result],
        incidentresponse_query_escalation_policy_users_on_call: typing.Callable[
            [], T_Result
        ],
        notifications_clear_message: typing.Callable[[], T_Result],
        notifications_create_message: typing.Callable[[], T_Result],
        notifications_get_message: typing.Callable[[], T_Result],
        siem_get_evidence: typing.Callable[[], T_Result],
        siem_get_investigation: typing.Callable[[], T_Result],
        siem_patch_investigation: typing.Callable[[], T_Result],
        siem_post_events: typing.Callable[[], T_Result],
        siem_query_alerts: typing.Callable[[], T_Result],
        siem_query_events: typing.Callable[[], T_Result],
        siem_query_investigations: typing.Callable[[], T_Result],
        siem_query_log_providers: typing.Callable[[], T_Result],
        sink_post_events: typing.Callable[[], T_Result],
        storage_delete_file: typing.Callable[[], T_Result],
        storage_download_file: typing.Callable[[], T_Result],
        storage_list_files: typing.Callable[[], T_Result],
        storage_upload_file: typing.Callable[[], T_Result],
        ticketing_create_attachment: typing.Callable[[], T_Result],
        ticketing_create_comment: typing.Callable[[], T_Result],
        ticketing_create_note: typing.Callable[[], T_Result],
        ticketing_create_ticket: typing.Callable[[], T_Result],
        ticketing_delete_attachment: typing.Callable[[], T_Result],
        ticketing_delete_comment: typing.Callable[[], T_Result],
        ticketing_delete_note: typing.Callable[[], T_Result],
        ticketing_download_attachment: typing.Callable[[], T_Result],
        ticketing_get_ticket: typing.Callable[[], T_Result],
        ticketing_list_attachments_metadata: typing.Callable[[], T_Result],
        ticketing_list_comments: typing.Callable[[], T_Result],
        ticketing_list_notes: typing.Callable[[], T_Result],
        ticketing_list_on_call: typing.Callable[[], T_Result],
        ticketing_list_projects: typing.Callable[[], T_Result],
        ticketing_list_remote_fields: typing.Callable[[], T_Result],
        ticketing_patch_note: typing.Callable[[], T_Result],
        ticketing_patch_ticket: typing.Callable[[], T_Result],
        ticketing_query_escalation_policies: typing.Callable[[], T_Result],
        ticketing_query_tickets: typing.Callable[[], T_Result],
        vulnerabilities_create_asset: typing.Callable[[], T_Result],
        vulnerabilities_create_findings: typing.Callable[[], T_Result],
        vulnerabilities_get_labels: typing.Callable[[], T_Result],
        vulnerabilities_get_scan_activity: typing.Callable[[], T_Result],
        vulnerabilities_get_scan_status: typing.Callable[[], T_Result],
        vulnerabilities_query_assets: typing.Callable[[], T_Result],
        vulnerabilities_query_findings: typing.Callable[[], T_Result],
        vulnerabilities_query_scans: typing.Callable[[], T_Result],
        vulnerabilities_update_asset: typing.Callable[[], T_Result],
        vulnerabilities_update_finding: typing.Callable[[], T_Result],
        vulnerabilities_upload_scan: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is OperationId.APPSEC_GET_APPLICATION_FINDING_DETAILS:
            return appsec_get_application_finding_details()
        if self is OperationId.APPSEC_QUERY_APPLICATION_FINDINGS:
            return appsec_query_application_findings()
        if self is OperationId.APPSEC_QUERY_APPLICATIONS:
            return appsec_query_applications()
        if self is OperationId.APPSEC_QUERY_FINDINGS:
            return appsec_query_findings()
        if self is OperationId.ASSETS_CREATE_ASSET:
            return assets_create_asset()
        if self is OperationId.ASSETS_GET_LABELS:
            return assets_get_labels()
        if self is OperationId.ASSETS_QUERY_DEVICES:
            return assets_query_devices()
        if self is OperationId.CLOUDSECURITY_QUERY_CLOUD_RESOURCE_INVENTORY:
            return cloudsecurity_query_cloud_resource_inventory()
        if self is OperationId.CLOUDSECURITY_QUERY_COMPLIANCE_FINDINGS:
            return cloudsecurity_query_compliance_findings()
        if self is OperationId.CLOUDSECURITY_QUERY_EVENTS:
            return cloudsecurity_query_events()
        if self is OperationId.CLOUDSECURITY_QUERY_IOMS:
            return cloudsecurity_query_ioms()
        if self is OperationId.CLOUDSECURITY_QUERY_THREATS:
            return cloudsecurity_query_threats()
        if self is OperationId.CUSTOM_DELETE:
            return custom_delete()
        if self is OperationId.CUSTOM_GET:
            return custom_get()
        if self is OperationId.CUSTOM_PATCH:
            return custom_patch()
        if self is OperationId.CUSTOM_POST:
            return custom_post()
        if self is OperationId.CUSTOM_POST_BATCH:
            return custom_post_batch()
        if self is OperationId.CUSTOM_QUERY:
            return custom_query()
        if self is OperationId.EDR_CREATE_IOCS:
            return edr_create_iocs()
        if self is OperationId.EDR_DELETE_IOCS:
            return edr_delete_iocs()
        if self is OperationId.EDR_GET_ENDPOINT:
            return edr_get_endpoint()
        if self is OperationId.EDR_NETWORK_QUARANTINE:
            return edr_network_quarantine()
        if self is OperationId.EDR_QUERY_ALERTS:
            return edr_query_alerts()
        if self is OperationId.EDR_QUERY_APPLICATIONS:
            return edr_query_applications()
        if self is OperationId.EDR_QUERY_EDR_EVENTS:
            return edr_query_edr_events()
        if self is OperationId.EDR_QUERY_ENDPOINTS:
            return edr_query_endpoints()
        if self is OperationId.EDR_QUERY_IOCS:
            return edr_query_iocs()
        if self is OperationId.EDR_QUERY_POSTURE_SCORE:
            return edr_query_posture_score()
        if self is OperationId.EDR_QUERY_THREATEVENTS:
            return edr_query_threatevents()
        if self is OperationId.IDENTITY_DISABLE_USER:
            return identity_disable_user()
        if self is OperationId.IDENTITY_ENABLE_USER:
            return identity_enable_user()
        if self is OperationId.IDENTITY_EXPIRE_ALL_USER_SESSIONS:
            return identity_expire_all_user_sessions()
        if self is OperationId.IDENTITY_FORCE_USER_PASSWORD_RESET:
            return identity_force_user_password_reset()
        if self is OperationId.IDENTITY_GET_GROUP:
            return identity_get_group()
        if self is OperationId.IDENTITY_GET_GROUP_MEMBERS:
            return identity_get_group_members()
        if self is OperationId.IDENTITY_GET_USER:
            return identity_get_user()
        if self is OperationId.IDENTITY_GET_USER_PICTURE:
            return identity_get_user_picture()
        if self is OperationId.IDENTITY_QUERY_AUDIT_LOG:
            return identity_query_audit_log()
        if self is OperationId.IDENTITY_QUERY_GROUPS:
            return identity_query_groups()
        if self is OperationId.IDENTITY_QUERY_USERS:
            return identity_query_users()
        if self is OperationId.INCIDENTRESPONSE_QUERY_ESCALATION_POLICIES:
            return incidentresponse_query_escalation_policies()
        if self is OperationId.INCIDENTRESPONSE_QUERY_ESCALATION_POLICY_USERS_ON_CALL:
            return incidentresponse_query_escalation_policy_users_on_call()
        if self is OperationId.NOTIFICATIONS_CLEAR_MESSAGE:
            return notifications_clear_message()
        if self is OperationId.NOTIFICATIONS_CREATE_MESSAGE:
            return notifications_create_message()
        if self is OperationId.NOTIFICATIONS_GET_MESSAGE:
            return notifications_get_message()
        if self is OperationId.SIEM_GET_EVIDENCE:
            return siem_get_evidence()
        if self is OperationId.SIEM_GET_INVESTIGATION:
            return siem_get_investigation()
        if self is OperationId.SIEM_PATCH_INVESTIGATION:
            return siem_patch_investigation()
        if self is OperationId.SIEM_POST_EVENTS:
            return siem_post_events()
        if self is OperationId.SIEM_QUERY_ALERTS:
            return siem_query_alerts()
        if self is OperationId.SIEM_QUERY_EVENTS:
            return siem_query_events()
        if self is OperationId.SIEM_QUERY_INVESTIGATIONS:
            return siem_query_investigations()
        if self is OperationId.SIEM_QUERY_LOG_PROVIDERS:
            return siem_query_log_providers()
        if self is OperationId.SINK_POST_EVENTS:
            return sink_post_events()
        if self is OperationId.STORAGE_DELETE_FILE:
            return storage_delete_file()
        if self is OperationId.STORAGE_DOWNLOAD_FILE:
            return storage_download_file()
        if self is OperationId.STORAGE_LIST_FILES:
            return storage_list_files()
        if self is OperationId.STORAGE_UPLOAD_FILE:
            return storage_upload_file()
        if self is OperationId.TICKETING_CREATE_ATTACHMENT:
            return ticketing_create_attachment()
        if self is OperationId.TICKETING_CREATE_COMMENT:
            return ticketing_create_comment()
        if self is OperationId.TICKETING_CREATE_NOTE:
            return ticketing_create_note()
        if self is OperationId.TICKETING_CREATE_TICKET:
            return ticketing_create_ticket()
        if self is OperationId.TICKETING_DELETE_ATTACHMENT:
            return ticketing_delete_attachment()
        if self is OperationId.TICKETING_DELETE_COMMENT:
            return ticketing_delete_comment()
        if self is OperationId.TICKETING_DELETE_NOTE:
            return ticketing_delete_note()
        if self is OperationId.TICKETING_DOWNLOAD_ATTACHMENT:
            return ticketing_download_attachment()
        if self is OperationId.TICKETING_GET_TICKET:
            return ticketing_get_ticket()
        if self is OperationId.TICKETING_LIST_ATTACHMENTS_METADATA:
            return ticketing_list_attachments_metadata()
        if self is OperationId.TICKETING_LIST_COMMENTS:
            return ticketing_list_comments()
        if self is OperationId.TICKETING_LIST_NOTES:
            return ticketing_list_notes()
        if self is OperationId.TICKETING_LIST_ON_CALL:
            return ticketing_list_on_call()
        if self is OperationId.TICKETING_LIST_PROJECTS:
            return ticketing_list_projects()
        if self is OperationId.TICKETING_LIST_REMOTE_FIELDS:
            return ticketing_list_remote_fields()
        if self is OperationId.TICKETING_PATCH_NOTE:
            return ticketing_patch_note()
        if self is OperationId.TICKETING_PATCH_TICKET:
            return ticketing_patch_ticket()
        if self is OperationId.TICKETING_QUERY_ESCALATION_POLICIES:
            return ticketing_query_escalation_policies()
        if self is OperationId.TICKETING_QUERY_TICKETS:
            return ticketing_query_tickets()
        if self is OperationId.VULNERABILITIES_CREATE_ASSET:
            return vulnerabilities_create_asset()
        if self is OperationId.VULNERABILITIES_CREATE_FINDINGS:
            return vulnerabilities_create_findings()
        if self is OperationId.VULNERABILITIES_GET_LABELS:
            return vulnerabilities_get_labels()
        if self is OperationId.VULNERABILITIES_GET_SCAN_ACTIVITY:
            return vulnerabilities_get_scan_activity()
        if self is OperationId.VULNERABILITIES_GET_SCAN_STATUS:
            return vulnerabilities_get_scan_status()
        if self is OperationId.VULNERABILITIES_QUERY_ASSETS:
            return vulnerabilities_query_assets()
        if self is OperationId.VULNERABILITIES_QUERY_FINDINGS:
            return vulnerabilities_query_findings()
        if self is OperationId.VULNERABILITIES_QUERY_SCANS:
            return vulnerabilities_query_scans()
        if self is OperationId.VULNERABILITIES_UPDATE_ASSET:
            return vulnerabilities_update_asset()
        if self is OperationId.VULNERABILITIES_UPDATE_FINDING:
            return vulnerabilities_update_finding()
        if self is OperationId.VULNERABILITIES_UPLOAD_SCAN:
            return vulnerabilities_upload_scan()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from ...base.types.url_string import UrlString
import pydantic
from .ticket_type_id import TicketTypeId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Ticket(UncheckedBaseModel):
    """
    The Ticket object represents ticket in the customer's systems like Salesforce, jira etc.
    """

    src_url: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The url of a ticket in the ticket system.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    The title of the ticket.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The linked ticket type determines whether the ticket is internal or in an external ticketing system.
    """

    type_id: typing.Optional[TicketTypeId] = pydantic.Field(default=None)
    """
    The normalized identifier for the ticket type.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique ticket identifier like ticket id.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Display(UncheckedBaseModel):
    """
    The Display object contains information about the physical or virtual display connected to a computer system.
    """

    color_depth: typing.Optional[int] = pydantic.Field(default=None)
    """
    The numeric color depth.
    """

    physical_height: typing.Optional[int] = pydantic.Field(default=None)
    """
    The numeric physical height of display.
    """

    physical_orientation: typing.Optional[int] = pydantic.Field(default=None)
    """
    The numeric physical orientation of display.
    """

    physical_width: typing.Optional[int] = pydantic.Field(default=None)
    """
    The numeric physical width of display.
    """

    scale_factor: typing.Optional[int] = pydantic.Field(default=None)
    """
    The numeric scale factor of display.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .container import Container
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Response(UncheckedBaseModel):
    """
    The Response Elements object describes characteristics of an API response.
    """

    code: typing.Optional[int] = pydantic.Field(default=None)
    """
    The numeric response sent to a request.
    """

    containers: typing.Optional[typing.List[Container]] = pydantic.Field(default=None)
    """
    When working with containerized applications, the set of containers which write to the standard the output of a particular logging driver. For example, this may be the set of containers involved in handling api requests and responses for a containerized application.
    """

    data: typing.Optional[typing.Optional[typing.Any]] = pydantic.Field(default=None)
    """
    The additional data that is associated with the api response.
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error Code
    """

    error_message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error Message
    """

    flags: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of communication flags, normalized to the captions of the flag_ids values. In the case of 'Other', they are defined by the event source.
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the event/finding, as defined by the source.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ....v_1_3_0.detectionfinding.classes.types.detection_finding import (
    DetectionFinding as ocsf_v_1_3_0_detectionfinding_classes_types_detection_finding_DetectionFinding,
)

DetectionFinding = (
    ocsf_v_1_3_0_detectionfinding_classes_types_detection_finding_DetectionFinding
)



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from .messages_response import MessagesResponse
import pydantic
from .meta_response import MetaResponse
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ApiResponse(UncheckedBaseModel):
    messages: typing.Optional[MessagesResponse] = pydantic.Field(default=None)
    """
    Additional messages from the service response that may be helpful to the client.
    """

    meta: typing.Optional[MetaResponse] = pydantic.Field(default=None)
    """
    Various metadata about the results organized by group, then type, then field.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ....v_1_3_0.cloudactivity.classes.types.cloud_activity import (
    CloudActivity as ocsf_v_1_3_0_cloudactivity_classes_types_cloud_activity_CloudActivity,
)

CloudActivity = ocsf_v_1_3_0_cloudactivity_classes_types_cloud_activity_CloudActivity



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .malware_classification_ids import MalwareClassificationIds
import pydantic
from .cve import Cve
from .file import File
from ...base.types.file_path import FilePath
from .malware_severity_id import MalwareSeverityId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Malware(UncheckedBaseModel):
    """
    The Malware object describes the classification of known malicious software, which is intentionally designed to cause damage to a computer, server, client, or computer network.
    """

    classification_ids: typing.List[MalwareClassificationIds] = pydantic.Field()
    """
    The list of normalized identifiers of the malware classifications.
    """

    classifications: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of malware classifications, normalized to the captions of the <code>classification_ids</code> values. In the case of 'Other', they are defined by the event source.
    """

    cves: typing.Optional[typing.List[Cve]] = pydantic.Field(default=None)
    """
    The list of Common Vulnerabilities and Exposures (CVE) identifiers associated with the malware. Reference: <a target='_blank' href='https://cve.mitre.org/'>CVE</a>
    """

    files: typing.Optional[typing.List[File]] = pydantic.Field(default=None)
    """
    The list of file objects representing files that were identified as infected by the malware.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The malware name, as reported by the detection engine.
    """

    num_infected: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of files that were identified to be infected by the malware.
    """

    path: typing.Optional[FilePath] = pydantic.Field(default=None)
    """
    The filesystem path of the malware that was observed.
    """

    provider: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name or identifier of the security solution or service that provided the malware detection information.
    """

    severity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The severity of the malware, normalized to the captions of the <code>severity_id</code> values. In the case of 'Other', they are defined by the event source.
    """

    severity_id: typing.Optional[MalwareSeverityId] = pydantic.Field(default=None)
    """
    The normalized identifier of the malware severity.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique identifier for the specific malware instance, as assigned by the detection engine (e.g., virus signature ID or IPS rule ID).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .sentinel_credential import SentinelCredential
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class SiemSentinel(UncheckedBaseModel):
    """
    Configuration for Microsoft Sentinel SIEM Product.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/microsoft-sentinel-siem-setup)
    """

    credential: SentinelCredential
    default_tables: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The default tables to use for queries. Supply this value if you would like to a subset of the default tables or non-ASIM data tables with Sentinel/Log Analytics queries.
    If more than one table is specified, a union operator will join them to query all of the tables at once. Supply a single value with `*` if you would like to query all tables without the normalizing ASIM transformations.
    **Note** that a single `*` entry will map to a `union *` query. Relying heavily on these queries is generally discouraged by Sentinel because they are slower and more resource intensive.
    """

    ingest_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Either the logs ingestion API url for you Data Collection Rule or your Data Collection Endpoint URL. This value must be supplied to ingest data into Microsoft Sentinel. This should look something like https://mydcr-xxx-westus2.logs.z1.ingest.monitor.azure.com
    """

    logs_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Base URL for the Microsoft Azure Monitor Logs API. Should only be supplied if using an alternate Microsoft cloud, such as GovCloud.
    """

    management_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Base URL for the Microsoft Azure Management API. Should only be supplied if using an alternate Microsoft cloud, such as GovCloud.
    """

    resource_group: str = pydantic.Field()
    """
    Azure resource group name that contains the Microsoft Sentinel workspace.
    """

    rule_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Immutable ID of the Data Collection Rule. This value must be supplied to ingest data into Microsoft Sentinel.
    """

    stream_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the Data Collection Rule stream. This value must be supplied to ingest data into Microsoft Sentinel.
    """

    subscription_id: str = pydantic.Field()
    """
    Azure subscription ID that contains the Microsoft Sentinel workspace.
    """

    tenant_id: str = pydantic.Field()
    """
    Azure Active Directory tenant ID that contains the Microsoft Sentinel workspace.
    """

    workspace_id: str = pydantic.Field()
    """
    ID of the Microsoft Sentinel Log Analytics workspace.
    """

    workspace_name: str = pydantic.Field()
    """
    Name of the Microsoft Sentinel Log Analytics workspace.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...ocsf.v_1_3_0.detectionfinding.classes.types.detection_finding import (
    DetectionFinding,
)

ThreatEvent = DetectionFinding



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .device import Device
import pydantic
from ...base.types.timestamp import Timestamp
import datetime as dt
from .product import Product
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Logger(UncheckedBaseModel):
    """
    The Logger object represents the device and product where events are stored with times for receipt and transmission. This may be at the source device where the event occurred, a remote scanning device, intermediate hops, or the ultimate destination.
    """

    device: typing.Optional[Device] = pydantic.Field(default=None)
    """
    The device where the events are logged.
    """

    event_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the event assigned by the logger.
    """

    log_level: typing.Optional[str] = pydantic.Field(default=None)
    """
    The audit level at which an event was generated.
    """

    log_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event log name. For example, syslog file name or Windows logging subsystem: Security.
    """

    log_provider: typing.Optional[str] = pydantic.Field(default=None)
    """
    The logging provider or logging service that logged the event. For example, Microsoft-Windows-Security-Auditing.
    """

    log_version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event log schema version that specifies the format of the original event. For example syslog version or Cisco Log Schema Version.
    """

    logged_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    <p>The time when the logging system collected and logged the event.</p>This attribute is distinct from the event time in that event time typically contain the time extracted from the original event. Most of the time, these two times will be different.
    """

    logged_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    <p>The time when the logging system collected and logged the event.</p>This attribute is distinct from the event time in that event time typically contain the time extracted from the original event. Most of the time, these two times will be different.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the logging product instance.
    """

    product: typing.Optional[Product] = pydantic.Field(default=None)
    """
    The product logging the event. This may be the event source product, a management server product, a scanning product, a SIEM, etc.
    """

    transmit_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the event was transmitted from the logging device to it's next destination.
    """

    transmit_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the event was transmitted from the logging device to it's next destination.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the logging product instance.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The version of the logging product.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from .meta_api_primary_response import MetaApiPrimaryResponse
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class MetaApiResponse(UncheckedBaseModel):
    primary: typing.Optional[MetaApiPrimaryResponse] = None
    list_: typing.Optional[typing.Dict[str, typing.List[str]]] = pydantic.Field(
        alias="list", default=None
    )
    """
    All responses from backing API calls, indexed by endpoint URL.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...common.types.base import Base
from .custom_id import CustomId
import pydantic
from ...organization_base.types.organization_id import OrganizationId
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class Custom(Base):
    id: CustomId
    fullname: str = pydantic.Field()
    """
    Human friendly display name for this custom provider.
    """

    organization_id: OrganizationId = pydantic.Field()
    """
    Organization that manages this custom provider.
    """

    data: str = pydantic.Field()
    """
    The YAML custom provider definition. Must follow the custom provider definition schema.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import datetime as dt
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class TimeseriesResult(UncheckedBaseModel):
    """
    Status timeseries object
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Interval time
    """

    succeeded: int = pydantic.Field()
    """
    Succeeded count
    """

    failed: int = pydantic.Field()
    """
    Failed count
    """

    cpu_time: int = pydantic.Field()
    """
    Cpu time in microseconds
    """

    in_bytes: int = pydantic.Field()
    """
    API input byte count
    """

    out_bytes: int = pydantic.Field()
    """
    API output byte count
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class CredentialType(str, enum.Enum):
    AWS = "aws"
    TOKEN = "token"
    BASIC = "basic"
    SECRET = "secret"
    O_AUTH_CLIENT = "o_auth_client"
    _UNKNOWN = "__CREDENTIALTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "CredentialType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        aws: typing.Callable[[], T_Result],
        token: typing.Callable[[], T_Result],
        basic: typing.Callable[[], T_Result],
        secret: typing.Callable[[], T_Result],
        o_auth_client: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is CredentialType.AWS:
            return aws()
        if self is CredentialType.TOKEN:
            return token()
        if self is CredentialType.BASIC:
            return basic()
        if self is CredentialType.SECRET:
            return secret()
        if self is CredentialType.O_AUTH_CLIENT:
            return o_auth_client()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .sentinel_one_credential import SentinelOneCredential
import typing
from .sentinel_one_edr_events_credential import SentinelOneEdrEventsCredential
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class EdrSentinelOne(UncheckedBaseModel):
    """
    Configuration for SentinelOne Singularity™ Endpoint.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/sentinelone-setup)
    """

    credential: SentinelOneCredential
    edr_events_credential: typing.Optional[SentinelOneEdrEventsCredential] = (
        pydantic.Field(default=None)
    )
    """
    Credential used for the SentinelOne Singularity Data Lake API. This credential is required when querying EDR events.
    """

    edr_events_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Base URL for the SentinelOne Singularity Data Lake API. This URL is required is required when querying EDR events.
    """

    skip_tls_verify: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When true, skips verification of the SentinelOne TLS certificate.
    """

    url: str = pydantic.Field()
    """
    Base URL for the SentinelOne Management API.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    CreateCustomRequest,
    CreateCustomResponse,
    CreateCustomResponseResult,
    Custom,
    CustomId,
    GetCustomResponse,
    ListCustomsResponse,
    PatchCustomResponse,
    UpdateCustomRequest,
    UpdateCustomResponse,
)

__all__ = [
    "CreateCustomRequest",
    "CreateCustomResponse",
    "CreateCustomResponseResult",
    "Custom",
    "CustomId",
    "GetCustomResponse",
    "ListCustomsResponse",
    "PatchCustomResponse",
    "UpdateCustomRequest",
    "UpdateCustomResponse",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from .asset import Asset
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class UploadScanRequest(UncheckedBaseModel):
    assets: typing.List[Asset] = pydantic.Field()
    """
    Assets and optional findings to upload in the vulnerability scanning system.
    """

    source_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the source for this scan, such as a tool or process. Does not have to be unique, defaults to `Integration import`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .group import Group
from ...base.types.hostname import Hostname
from .user import User
from .graph import Graph
from .application_risk_level_id import ApplicationRiskLevelId
from .sbom import Sbom
from .key_value_object import KeyValueObject
from .url import Url
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Application(UncheckedBaseModel):
    """
    An Application describes the details for an inventoried application as reported by an Application Security tool or other Developer-centric tooling. Applications can be defined as Kubernetes resources, Containerized resources, or application hosting-specific cloud sources such as AWS Elastic BeanStalk, AWS Lightsail, or Azure Logic Apps.
    """

    criticality: typing.Optional[str] = pydantic.Field(default=None)
    """
    The criticality of the application as defined by the event source.
    """

    data: typing.Optional[typing.Optional[typing.Any]] = pydantic.Field(default=None)
    """
    Additional data describing the application.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    A description or commentary for an application, usually retrieved from an upstream system.
    """

    group: typing.Optional[Group] = pydantic.Field(default=None)
    """
    The name of the related application or associated resource group.
    """

    hostname: typing.Optional[Hostname] = pydantic.Field(default=None)
    """
    The fully qualified name of the application.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of labels associated to the application.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the application.
    """

    owner: typing.Optional[User] = pydantic.Field(default=None)
    """
    The identity of the service or user account that owns the application.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    The cloud region of the resource.
    """

    resource_relationship: typing.Optional[Graph] = pydantic.Field(default=None)
    """
    A graph representation showing how this application relates to and interacts with other entities in the environment. This can include parent/child relationships, dependencies, or other connections.
    """

    risk_level: typing.Optional[str] = pydantic.Field(default=None)
    """
    The risk level, normalized to the caption of the risk_level_id value.
    """

    risk_level_id: typing.Optional[ApplicationRiskLevelId] = pydantic.Field(
        default=None
    )
    """
    The normalized risk level id.
    """

    risk_score: typing.Optional[int] = pydantic.Field(default=None)
    """
    The risk score as reported by the event source.
    """

    sbom: typing.Optional[Sbom] = pydantic.Field(default=None)
    """
    The Software Bill of Materials (SBOM) associated with the application
    """

    tags: typing.Optional[typing.List[KeyValueObject]] = pydantic.Field(default=None)
    """
    The list of tags; <code>{key:value}</code> pairs associated to the application.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of application as defined by the event source, e.g., <code>GitHub</code>, <code>Azure Logic App</code>, or <code>Amazon Elastic BeanStalk</code>.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier for the application.
    """

    uid_alt: typing.Optional[str] = pydantic.Field(default=None)
    """
    An alternative or contextual identifier for the application, such as a configuration, organization, or license UID.
    """

    url: typing.Optional[Url] = pydantic.Field(default=None)
    """
    The URL of the application.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The semantic version of the application, e.g., <code>1.7.4</code>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .threat_actor_type_id import ThreatActorTypeId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class ThreatActor(UncheckedBaseModel):
    """
    Threat actor is responsible for the observed malicious activity.
    """

    name: str = pydantic.Field()
    """
    The name of the threat actor.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The classification of the threat actor based on their motivations, capabilities, or affiliations. Common types include nation-state actors, cybercriminal groups, hacktivists, or insider threats.
    """

    type_id: typing.Optional[ThreatActorTypeId] = pydantic.Field(default=None)
    """
    The normalized datastore resource type identifier.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from ...common.types.id import Id
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class GroupSummary(UncheckedBaseModel):
    """
    The group summary object gives a summary of a group in an escalation policy. A group is a collection of agents that can be notified when certain conditions are met
    """

    id: Id = pydantic.Field()
    """
    Unique identifier for the group.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the group.
    """

    additional_properties: bool = pydantic.Field()
    """
    Indicates if the group has additional properties that can be retrieved.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AdhocRole,
    Resources,
    RoleAccounts,
    RoleId,
    RoleIntegrations,
    RoleName,
    RoleOrganizations,
)

__all__ = [
    "AdhocRole",
    "Resources",
    "RoleAccounts",
    "RoleId",
    "RoleIntegrations",
    "RoleName",
    "RoleOrganizations",
]



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class Extension(UncheckedBaseModel):
    """
    The OCSF Schema Extension object provides detailed information about the schema extension used to construct the event. The schema extensions are registered in the <a target='_blank' href='https://github.com/ocsf/ocsf-schema/blob/main/extensions.md'>extensions.md</a> file.
    """

    name: str = pydantic.Field()
    """
    The schema extension name. For example: <code>dev</code>.
    """

    uid: str = pydantic.Field()
    """
    The schema extension unique identifier. For example: <code>999</code>.
    """

    version: str = pydantic.Field()
    """
    The schema extension version. For example: <code>1.0.0-alpha.2</code>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .operation_frequency import OperationFrequency, OperationFrequency_Periodic
from .operation_frequency_periodic import OperationFrequencyPeriodic
from .operation_incremental_pull_config import OperationIncrementalPullConfig
from .operation_incremental_pull_mode import OperationIncrementalPullMode
from .operation_schedule import OperationSchedule
from .scheduled_operation_id import ScheduledOperationId

__all__ = [
    "OperationFrequency",
    "OperationFrequencyPeriodic",
    "OperationFrequency_Periodic",
    "OperationIncrementalPullConfig",
    "OperationIncrementalPullMode",
    "OperationSchedule",
    "ScheduledOperationId",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .ping_one_auth_url import PingOneAuthUrl
import pydantic
from .ping_one_credential import PingOneCredential
from .ping_one_apiurl import PingOneApiurl
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class IdentityPingOne(UncheckedBaseModel):
    """
    Configuration for PingOne Cloud Platform.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/ping-identity-setup)
    """

    auth_url: PingOneAuthUrl = pydantic.Field()
    """
    Base URL for making authentication requests to PingOne.
    """

    client_id: str = pydantic.Field()
    """
    Client ID for the application set up as a worker.
    """

    credential: PingOneCredential
    organization_id: str = pydantic.Field()
    """
    The organization ID that the client app is a part of.
    """

    url: PingOneApiurl = pydantic.Field()
    """
    Base URL for the PingOne API.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..events.types.event import Event
from ..core.request_options import RequestOptions
from .types.create_sink_events_response import CreateSinkEventsResponse
from ..core.unchecked_base_model import construct_type
from ..common.errors.bad_request_error import BadRequestError
from ..common.types.problem import Problem
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.internal_server_error import InternalServerError
from ..common.errors.not_implemented_error import NotImplementedError
from ..common.errors.bad_gateway_error import BadGatewayError
from ..common.errors.service_unavailable_error import ServiceUnavailableError
from ..common.errors.gateway_timeout_error import GatewayTimeoutError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SinkClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def post_events(
        self,
        *,
        request: typing.Sequence[Event],
        location: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSinkEventsResponse:
        """
        Writes a batch of `Event` objects to the Sink configured with the token used for authentication.

        Parameters
        ----------
        request : typing.Sequence[Event]

        location : typing.Optional[str]
            Updates the path or queue name for the sink. If not provided, the default path or queue name is used. When provided, the path or queue name is appended to the default path or queue name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSinkEventsResponse

        Examples
        --------
        import datetime

        from synqly import SynqlyEngine
        from synqly.events import Event_AccountChange
        from synqly.ocsf.v_1_3_0.accountchange.objects import (
            Account,
            Actor,
            Api,
            AutonomousSystem,
            Cloud,
            Container,
            Device,
            DeviceHwInfo,
            Enrichment,
            Extension,
            Feature,
            Group,
            HttpRequest,
            Idp,
            Image,
            Location,
            Logger,
            Metadata,
            NetworkEndpoint,
            NetworkProxy,
            Observable,
            Organization,
            Os,
            Osint,
            Policy,
            Process,
            Product,
            Request,
            Response,
            Service,
            Session,
            Url,
            User,
        )

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.sink.post_events(
            location="string",
            request=[
                Event_AccountChange(
                    activity_id=1,
                    activity_name="string",
                    actor=Actor(
                        actor_type="string",
                        actor_type_id=1,
                        app_name="string",
                        app_uid="string",
                        authorizations=[],
                        groups=[],
                        idp=Idp(),
                        invoked_by="string",
                        process=Process(),
                        session=Session(),
                        user=User(),
                    ),
                    api=Api(
                        group=Group(),
                        operation="string",
                        request=Request(
                            uid="string",
                        ),
                        response=Response(),
                        service=Service(),
                        version="string",
                    ),
                    category_name="string",
                    category_uid=1,
                    class_uid=1,
                    cloud=Cloud(
                        account=Account(),
                        org=Organization(),
                        project_uid="string",
                        provider="string",
                        region="string",
                        zone="string",
                    ),
                    count=1,
                    custom_fields={"string": {"key": "value"}},
                    device=Device(
                        agent_list=[],
                        autoscale_uid="string",
                        boot_time=1,
                        boot_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        container=Container(),
                        created_time=1,
                        created_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        desc="string",
                        domain="string",
                        first_seen_time=1,
                        first_seen_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        groups=[],
                        hostname="string",
                        hw_info=DeviceHwInfo(),
                        hypervisor="string",
                        image=Image(
                            uid="string",
                        ),
                        imei="string",
                        instance_uid="string",
                        interface_name="string",
                        interface_uid="string",
                        ip="string",
                        ip_addresses=[],
                        is_compliant=True,
                        is_managed=True,
                        is_personal=True,
                        is_trusted=True,
                        last_seen_time=1,
                        last_seen_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        location=Location(),
                        mac="string",
                        mac_addresses=[],
                        modified_time=1,
                        modified_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        name="string",
                        namespace_pid=1,
                        netbios_names=[],
                        network_interfaces=[],
                        network_status="string",
                        network_status_id=1,
                        org=Organization(),
                        os=Os(
                            name="string",
                            type_id=1,
                        ),
                        owner=User(),
                        region="string",
                        risk_level="string",
                        risk_level_id=1,
                        risk_score=1,
                        subnet="string",
                        subnet_uid="string",
                        sw_info=[],
                        type="string",
                        type_id=1,
                        uid="string",
                        uid_alt="string",
                        vendor=Organization(),
                        vlan_uid="string",
                        vpc_uid="string",
                        zone="string",
                    ),
                    duration=1,
                    end_time=1,
                    end_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    enrichments=[
                        Enrichment(
                            data={"key": "value"},
                            name="string",
                            value="string",
                        )
                    ],
                    http_request=HttpRequest(
                        args="string",
                        http_headers=[],
                        http_method="string",
                        length=1,
                        referrer="string",
                        uid="string",
                        url=Url(),
                        user_agent="string",
                        user_agent_name="string",
                        version="string",
                        x_forwarded_for=[],
                    ),
                    message="string",
                    metadata=Metadata(
                        correlation_uid="string",
                        event_code="string",
                        extension=Extension(
                            name="string",
                            uid="string",
                            version="string",
                        ),
                        extensions=[
                            Extension(
                                name="string",
                                uid="string",
                                version="string",
                            )
                        ],
                        labels=["string"],
                        log_level="string",
                        log_name="string",
                        log_provider="string",
                        log_version="string",
                        logged_time=1,
                        logged_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        loggers=[Logger()],
                        modified_time=1,
                        modified_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        original_time="string",
                        processed_time=1,
                        processed_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        product=Product(
                            cpe_name="string",
                            distribution_mode="string",
                            feature=Feature(
                                name="string",
                                uid="string",
                                version="string",
                            ),
                            lang="string",
                            name="string",
                            path="string",
                            uid="string",
                            url_string="string",
                            vendor_name="string",
                            version="string",
                            workload="string",
                        ),
                        profiles=["string"],
                        sequence=1,
                        tenant_uid="string",
                        uid="string",
                        version="string",
                    ),
                    observables=[
                        Observable(
                            name="string",
                            type_id=1,
                        )
                    ],
                    osint=[
                        Osint(
                            type_id=1,
                            value="string",
                        )
                    ],
                    policy=Policy(
                        desc="string",
                        group=Group(),
                        is_applied=True,
                        name="string",
                        uid="string",
                        version="string",
                    ),
                    raw_data="string",
                    severity="string",
                    severity_id=1,
                    src_endpoint=NetworkEndpoint(
                        agent_list=[],
                        autonomous_system=AutonomousSystem(),
                        container=Container(),
                        domain="string",
                        hostname="string",
                        hw_info=DeviceHwInfo(),
                        instance_uid="string",
                        interface_name="string",
                        interface_uid="string",
                        intermediate_ips=[],
                        ip="string",
                        location=Location(),
                        mac="string",
                        name="string",
                        namespace_pid=1,
                        os=Os(
                            name="string",
                            type_id=1,
                        ),
                        owner=User(),
                        port=1,
                        proxy_endpoint=NetworkProxy(),
                        subnet_uid="string",
                        svc_name="string",
                        type="string",
                        type_id=1,
                        uid="string",
                        vlan_uid="string",
                        vpc_uid="string",
                        zone="string",
                    ),
                    start_time=1,
                    start_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    status="string",
                    status_code="string",
                    status_detail="string",
                    status_id=1,
                    time=1,
                    time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    timezone_offset=1,
                    type_name="string",
                    type_uid=1,
                    unmapped={"string": {"key": "value"}},
                    user=User(),
                    user_result=User(),
                )
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/sink/events",
            method="POST",
            params={
                "location": location,
            },
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateSinkEventsResponse,
                    construct_type(
                        type_=CreateSinkEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncSinkClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def post_events(
        self,
        *,
        request: typing.Sequence[Event],
        location: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSinkEventsResponse:
        """
        Writes a batch of `Event` objects to the Sink configured with the token used for authentication.

        Parameters
        ----------
        request : typing.Sequence[Event]

        location : typing.Optional[str]
            Updates the path or queue name for the sink. If not provided, the default path or queue name is used. When provided, the path or queue name is appended to the default path or queue name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSinkEventsResponse

        Examples
        --------
        import asyncio
        import datetime

        from synqly import AsyncSynqlyEngine
        from synqly.events import Event_AccountChange
        from synqly.ocsf.v_1_3_0.accountchange.objects import (
            Account,
            Actor,
            Api,
            AutonomousSystem,
            Cloud,
            Container,
            Device,
            DeviceHwInfo,
            Enrichment,
            Extension,
            Feature,
            Group,
            HttpRequest,
            Idp,
            Image,
            Location,
            Logger,
            Metadata,
            NetworkEndpoint,
            NetworkProxy,
            Observable,
            Organization,
            Os,
            Osint,
            Policy,
            Process,
            Product,
            Request,
            Response,
            Service,
            Session,
            Url,
            User,
        )

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.sink.post_events(
                location="string",
                request=[
                    Event_AccountChange(
                        activity_id=1,
                        activity_name="string",
                        actor=Actor(
                            actor_type="string",
                            actor_type_id=1,
                            app_name="string",
                            app_uid="string",
                            authorizations=[],
                            groups=[],
                            idp=Idp(),
                            invoked_by="string",
                            process=Process(),
                            session=Session(),
                            user=User(),
                        ),
                        api=Api(
                            group=Group(),
                            operation="string",
                            request=Request(
                                uid="string",
                            ),
                            response=Response(),
                            service=Service(),
                            version="string",
                        ),
                        category_name="string",
                        category_uid=1,
                        class_uid=1,
                        cloud=Cloud(
                            account=Account(),
                            org=Organization(),
                            project_uid="string",
                            provider="string",
                            region="string",
                            zone="string",
                        ),
                        count=1,
                        custom_fields={"string": {"key": "value"}},
                        device=Device(
                            agent_list=[],
                            autoscale_uid="string",
                            boot_time=1,
                            boot_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            container=Container(),
                            created_time=1,
                            created_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            desc="string",
                            domain="string",
                            first_seen_time=1,
                            first_seen_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            groups=[],
                            hostname="string",
                            hw_info=DeviceHwInfo(),
                            hypervisor="string",
                            image=Image(
                                uid="string",
                            ),
                            imei="string",
                            instance_uid="string",
                            interface_name="string",
                            interface_uid="string",
                            ip="string",
                            ip_addresses=[],
                            is_compliant=True,
                            is_managed=True,
                            is_personal=True,
                            is_trusted=True,
                            last_seen_time=1,
                            last_seen_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            location=Location(),
                            mac="string",
                            mac_addresses=[],
                            modified_time=1,
                            modified_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            name="string",
                            namespace_pid=1,
                            netbios_names=[],
                            network_interfaces=[],
                            network_status="string",
                            network_status_id=1,
                            org=Organization(),
                            os=Os(
                                name="string",
                                type_id=1,
                            ),
                            owner=User(),
                            region="string",
                            risk_level="string",
                            risk_level_id=1,
                            risk_score=1,
                            subnet="string",
                            subnet_uid="string",
                            sw_info=[],
                            type="string",
                            type_id=1,
                            uid="string",
                            uid_alt="string",
                            vendor=Organization(),
                            vlan_uid="string",
                            vpc_uid="string",
                            zone="string",
                        ),
                        duration=1,
                        end_time=1,
                        end_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        enrichments=[
                            Enrichment(
                                data={"key": "value"},
                                name="string",
                                value="string",
                            )
                        ],
                        http_request=HttpRequest(
                            args="string",
                            http_headers=[],
                            http_method="string",
                            length=1,
                            referrer="string",
                            uid="string",
                            url=Url(),
                            user_agent="string",
                            user_agent_name="string",
                            version="string",
                            x_forwarded_for=[],
                        ),
                        message="string",
                        metadata=Metadata(
                            correlation_uid="string",
                            event_code="string",
                            extension=Extension(
                                name="string",
                                uid="string",
                                version="string",
                            ),
                            extensions=[
                                Extension(
                                    name="string",
                                    uid="string",
                                    version="string",
                                )
                            ],
                            labels=["string"],
                            log_level="string",
                            log_name="string",
                            log_provider="string",
                            log_version="string",
                            logged_time=1,
                            logged_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            loggers=[Logger()],
                            modified_time=1,
                            modified_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            original_time="string",
                            processed_time=1,
                            processed_time_dt=datetime.datetime.fromisoformat(
                                "2024-01-15 09:30:00+00:00",
                            ),
                            product=Product(
                                cpe_name="string",
                                distribution_mode="string",
                                feature=Feature(
                                    name="string",
                                    uid="string",
                                    version="string",
                                ),
                                lang="string",
                                name="string",
                                path="string",
                                uid="string",
                                url_string="string",
                                vendor_name="string",
                                version="string",
                                workload="string",
                            ),
                            profiles=["string"],
                            sequence=1,
                            tenant_uid="string",
                            uid="string",
                            version="string",
                        ),
                        observables=[
                            Observable(
                                name="string",
                                type_id=1,
                            )
                        ],
                        osint=[
                            Osint(
                                type_id=1,
                                value="string",
                            )
                        ],
                        policy=Policy(
                            desc="string",
                            group=Group(),
                            is_applied=True,
                            name="string",
                            uid="string",
                            version="string",
                        ),
                        raw_data="string",
                        severity="string",
                        severity_id=1,
                        src_endpoint=NetworkEndpoint(
                            agent_list=[],
                            autonomous_system=AutonomousSystem(),
                            container=Container(),
                            domain="string",
                            hostname="string",
                            hw_info=DeviceHwInfo(),
                            instance_uid="string",
                            interface_name="string",
                            interface_uid="string",
                            intermediate_ips=[],
                            ip="string",
                            location=Location(),
                            mac="string",
                            name="string",
                            namespace_pid=1,
                            os=Os(
                                name="string",
                                type_id=1,
                            ),
                            owner=User(),
                            port=1,
                            proxy_endpoint=NetworkProxy(),
                            subnet_uid="string",
                            svc_name="string",
                            type="string",
                            type_id=1,
                            uid="string",
                            vlan_uid="string",
                            vpc_uid="string",
                            zone="string",
                        ),
                        start_time=1,
                        start_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        status="string",
                        status_code="string",
                        status_detail="string",
                        status_id=1,
                        time=1,
                        time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        timezone_offset=1,
                        type_name="string",
                        type_uid=1,
                        unmapped={"string": {"key": "value"}},
                        user=User(),
                        user_result=User(),
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/sink/events",
            method="POST",
            params={
                "location": location,
            },
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateSinkEventsResponse,
                    construct_type(
                        type_=CreateSinkEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
